@@ -105,19 +105,8 @@ sub OutputError($)
 sub OutputStat
 {
 	my ($self, $hash ) = @_;
-
 	foreach my $key ( sort keys %$hash )
 	{
-	    if ( ref( $hash->{ $key } ) )
-	    {
-		# Nested element
-		print( "... $key\n" );
-		foreach my $item ( @{ $hash->{ $key } } )
-		{
-		    print( "... ... $item\n" );
-		}
-	    }
-
 	    print("... $key ", $hash->{ $key }, "\n" );
 	}
 	print("\n");
@@ -141,30 +130,6 @@ sub Prompt($)
 	return <>;
 }
 
-
-#
-# Function to diff two files. This default implementation does nothing of
-# great use. Users should consider using the Algorithm::Diff module to get
-# real diff listings.
-#
-sub Diff($$$$)
-{
-    my $self 		= shift;
-    my $f1		= shift;
-    my $f2		= shift;
-    my $diffFlags	= shift;
-    my $differ		= shift;
-
-    if ( $differ )
-    {
-	print( "$f1 and $f2 differ\n" );
-    }
-    else
-    {
-	print( "$f1 and $f2 are the same\n" );
-    }
-}
-
 1;
 
 __END__
@@ -244,30 +209,18 @@ derive a subclass from P4::UI and override the appropriate methods.
 =item C<OutputStat( $hashref )>
 
 	Print the output of a command in tagged format. The tagged
-	data is passed as a hash reference for ease of use. Multi 
-	line fields (such as mappings) are presented as array
-	references within the hash. i.e. in a clientspec, the 
-	View field in the form is normally transmitted by Perforce 
-	as a sequence of fields: "View0", "View1", etc. P4::UI gets
-	these as a single "View" member of the hash which is itself 
-	an array reference containing the view records in order.
+	data is passed as a hash reference for ease of use.
 
 =item C<OutputText( $text, $length )>
 
 	Prints $length bytes of $text on STDOUT
 
+
 =item C<Prompt( $prompt )>
 
 	Prints the prompt string $prompt and then reads a line
 	of input from the user returning the input line.
 
-=item C<Diff( $f1, $f2, $flags, $differ )>
-
-	Diff two files. The default implementation of this method
-	does nothing of any great use. It's intended to be overridden
-	in subclasses.
-	
-
 =back
 
 =head1 LICENCE
